WITH   Ada.Unchecked_Conversion ;
WITH   Interfaces.C ;
WITH   Stdarg ;
WITH   System ;
WITH   Win32 ;
WITH   Win32.Crt.Strings ;
WITH   Win32.Windef ;
WITH   Win32.Wingdi ;
WITH   Win32.Winuser ;

PACKAGE BODY   PMPrime_Pkg  IS

   FUNCTION   CP   ( I_Str : Win32.CHAR_Array )  RETURN  Win32.LPCSTR  IS
      FUNCTION   UC  IS
         new Ada.Unchecked_Conversion ( System.Address , Win32.LPCSTR );
   BEGIN
      RETURN  UC ( I_Str(I_Str'First)'Address );
   END CP;

   FUNCTION   CPS   ( I_Str : Win32.CHAR_Array )  RETURN  Win32.LPSTR  IS
      FUNCTION   UC  IS
         new Ada.Unchecked_Conversion ( System.Address , Win32.LPSTR );
   BEGIN
      RETURN  UC ( I_Str(I_Str'First)'Address );
   END CPS;

   FUNCTION   To_IntW  IS  new Ada.Unchecked_Conversion(Win32.PCHAR, Integer);
   FUNCTION   To_PCHAR  IS  new Ada.Unchecked_Conversion(Integer, Win32.PCHAR);

   USE TYPE   Win32.CHAR_Array ;      --- To get "&"

   Nul : Win32.Char RENAMES Win32.Nul ;

   IgnoredI : Win32.INT ;
   IgnoredP : Win32.PSTR ;

   C_MainPMTitle : CONSTANT Win32.CHAR_Array := "Information"  &  Nul ;
   C_MainPMMsg   : CONSTANT Win32.CHAR_Array := "***** PRIME NUMBERS: "  &  Nul ;
   C_MainNbFmt   : CONSTANT Win32.CHAR_Array := "%ld"  &  Nul ;
   C_SpaceZ : CONSTANT Win32.CHAR_Array := " "  &  Nul ;

   L_Buffer : ALIASED Win32.CHAR_Array(0..1024);
   L_Buf25  : ALIASED Win32.CHAR_Array(0..25);

   PROCEDURE   ShowPMPrime  IS
      FUNCTION   "&"  IS  new Stdarg.Concat ( Win32.LONG );

      IsPrimeArray : Array(1..1000) of Boolean ;
      j , jMax : Integer ;

      L_P : Win32.LPSTR ;
      L_TmpLen : Standard.INTEGER ;
   BEGIN
      L_P := CPS(L_Buffer);

      IgnoredP := Win32.Crt.Strings.StrCpy ( L_P , CP(C_MainPMMsg) );
      L_TmpLen := Standard.INTEGER( Win32.Crt.Strings.StrLen(Win32.PCCH(L_P)) );
      L_P      := To_PCHAR(To_IntW(L_P) + L_TmpLen);

      ----- LOOP 1 , On each number
      FOR i in 1..1000 LOOP

         ----- Could be a prime number
         IsPrimeArray(i) := TRUE ;

         j    := 2 ;
         jMax := Integer( i / 2 );

         ----- LOOP 2 , Search a divisor
         WHILE IsPrimeArray(i) and (j <= jMax) LOOP
            ----- "j" can divide "i" ?
            IF (i mod j) = 0 THEN
               IsPrimeArray(i) := False ;
            END IF;
            j := j + 1 ;
         END LOOP;

         ----- Display only prime numbers
         IF IsPrimeArray(i) = TRUE THEN
            IgnoredI := Win32.Winuser.wvsprintf ( CPS(L_Buf25) , CP(C_MainNbFmt)
                                                , Stdarg.Empty & Win32.LONG(i) );

            IgnoredP := Win32.Crt.Strings.StrCpy ( L_P , CP(L_Buf25) );
            L_TmpLen := Standard.INTEGER( Win32.Crt.Strings.StrLen(Win32.PCCH(L_P)) );
            L_P      := To_PCHAR(To_IntW(L_P) + L_TmpLen);

            IgnoredP := Win32.Crt.Strings.StrCpy ( L_P , CP(C_SpaceZ) );
            L_TmpLen := Standard.INTEGER( Win32.Crt.Strings.StrLen(Win32.PCCH(L_P)) );
            L_P      := To_PCHAR(To_IntW(L_P) + L_TmpLen);
         END IF;
      END LOOP;

      IgnoredI := Win32.Winuser.MessageBox ( System.Null_Address
                                           , CP(L_Buffer)
                                           , CP(C_MainPMTitle)
                                           , Win32.Winuser.MB_OK or Win32.Winuser.MB_ICONEXCLAMATION );
   END ShowPMPrime;

END PMPrime_Pkg;
