WITH   Ada.Unchecked_Conversion ;
WITH   Interfaces.C ;
WITH   Stdarg ;
WITH   System ;
WITH   System.Img_Real ;
WITH   Win32 ;
WITH   Win32.Crt.Strings ;
WITH   Win32.Windef ;
WITH   Win32.Wingdi ;
WITH   Win32.Winuser ;

PACKAGE BODY   PMCalc_Pkg  IS

   FUNCTION   SCP   ( I_Str : String )  RETURN  Win32.LPCSTR  IS
      FUNCTION   UC  IS
         new Ada.Unchecked_Conversion ( System.Address , Win32.LPCSTR );
   BEGIN
      RETURN  UC ( I_Str(I_Str'First)'Address );
   END SCP;

   FUNCTION   CP   ( I_Str : Win32.CHAR_Array )  RETURN  Win32.LPCSTR  IS
      FUNCTION   UC  IS
         new Ada.Unchecked_Conversion ( System.Address , Win32.LPCSTR );
   BEGIN
      RETURN  UC ( I_Str(I_Str'First)'Address );
   END CP;

   FUNCTION   CPS   ( I_Str : Win32.CHAR_Array )  RETURN  Win32.LPSTR  IS
      FUNCTION   UC  IS
         new Ada.Unchecked_Conversion ( System.Address , Win32.LPSTR );
   BEGIN
      RETURN  UC ( I_Str(I_Str'First)'Address );
   END CPS;

   FUNCTION   To_IntW  IS  new Ada.Unchecked_Conversion(Win32.PCHAR, Integer);
   FUNCTION   To_PCHAR  IS  new Ada.Unchecked_Conversion(Integer, Win32.PCHAR);

   FUNCTION   StPCpy   ( I_Dest : Win32.PSTR
                       ; I_Source : Win32.PCSTR )  RETURN  Win32.PSTR  IS
      L_IgnoredP : Win32.PSTR ;
      L_AddedLen : Standard.INTEGER ;
   BEGIN
      L_IgnoredP := Win32.Crt.Strings.StrCpy ( I_Dest , I_Source );
      L_AddedLen := Standard.INTEGER( Win32.Crt.Strings.StrLen(Win32.PCCH(I_Dest)) );
      RETURN  To_PCHAR(To_IntW(I_Dest) + L_AddedLen);
   END StPCPy;

   FUNCTION   StPCpyFloat   ( I_Dest : Win32.PCHAR
                            ; I_Number : Float )  RETURN  Win32.PSTR  IS
      USE TYPE   Interfaces.C.size_t ;

      L_IgnoredP : Win32.PSTR ;

      Default_Fore : Integer := 1 ;
      Default_Aft  : Integer := Float'Digits - 1 ;
      Default_Exp  : Integer := 3 ;

      L_StringBuf : String(1..100);
      L_StringCnt : Integer ;

      L_AddedLen : Standard.INTEGER ;
   BEGIN
      L_StringCnt := 0 ;
      System.Img_Real.Set_Image_Real ( Standard.LONG_LONG_FLOAT(I_Number)
                                     , L_StringBuf , L_StringCnt
                                     , Fore => Default_Fore , Aft => Default_Aft , Exp => Default_Exp );

      L_IgnoredP := Win32.Crt.Strings.StrNCat ( I_Dest , SCP(L_StringBuf) , Interfaces.C.size_t(L_StringCnt) );

      L_AddedLen := Standard.INTEGER( Win32.Crt.Strings.StrLen(Win32.PCCH(I_Dest)) );
      RETURN  To_PCHAR(To_IntW(I_Dest) + L_AddedLen);
   END StPCpyFloat;

   USE TYPE   Win32.CHAR_Array ;      --- To get "&"

   Nul : Win32.Char RENAMES Win32.Nul ;

   IgnoredI : Win32.INT ;

   C_MainPMTitle : CONSTANT Win32.CHAR_Array := "Information"  &  Nul ;
   C_sRadiusIs   : CONSTANT Win32.CHAR_Array := "Radius="  &  Nul ;
   C_sCircumfIs  : CONSTANT Win32.CHAR_Array := "Circumference="  &  Nul ;
   C_sAreaIs     : CONSTANT Win32.CHAR_Array := "Area="  &  Nul ;
   CRLFz : CONSTANT Win32.CHAR_Array := Win32.Char(Character'Val(13))  &  Win32.Char(Character'Val(10))  &  Nul ;

   L_Buffer : ALIASED Win32.CHAR_Array(0..255);

   PROCEDURE   ShowPMCalc  IS
      PI : CONSTANT Float := 3.14159 ;

      R : Float ;
      C , A : Float ;

      L_P : Win32.LPSTR ;
   BEGIN
      R := 10.0 ;

      C := 2.0 * PI * R ;
      A := PI * R * R ;

      L_Buffer(0) := Nul ;
      L_P := CPS(L_Buffer);

      L_P := StPCpy ( L_P , CP(C_sRadiusIs) );
      L_P := StPCpyFloat ( L_P , R );
      L_P := StPCpy ( L_P , CP(CRLFz) );

      L_P := StPCpy ( L_P , CP(C_sCircumfIs) );
      L_P := StPCpyFloat ( L_P , C );
      L_P := StPCpy ( L_P , CP(CRLFz) );

      L_P := StPCpy ( L_P , CP(C_sAreaIs) );
      L_P := StPCpyFloat ( L_P , A );
      L_P := StPCpy ( L_P , CP(CRLFz) );

      IgnoredI := Win32.Winuser.MessageBox ( System.Null_Address
                                           , CP(L_Buffer)
                                           , CP(C_MainPMTitle)
                                           , Win32.Winuser.MB_OK or Win32.Winuser.MB_ICONEXCLAMATION );
   END ShowPMCalc;

END PMCalc_Pkg;
