WITH   Ada.Unchecked_Conversion ;
WITH   Interfaces.C ;
WITH   Stdarg ;
WITH   System ;
WITH   System.Img_Real ;
WITH   Win32 ;
WITH   Win32.Crt.Strings ;
WITH   Win32.Windef ;
WITH   Win32.Wingdi ;
WITH   Win32.Winuser ;

PACKAGE BODY   PMCalc_Pkg  IS

   FUNCTION   SCP   ( I_Str : String )  RETURN  Win32.LPCSTR  IS
      FUNCTION   UC  IS
         new Ada.Unchecked_Conversion ( System.Address , Win32.LPCSTR );
   BEGIN
      RETURN  UC ( I_Str(I_Str'First)'Address );
   END SCP;

   FUNCTION   CP   ( I_Str : Win32.CHAR_Array )  RETURN  Win32.LPCSTR  IS
      FUNCTION   UC  IS
         new Ada.Unchecked_Conversion ( System.Address , Win32.LPCSTR );
   BEGIN
      RETURN  UC ( I_Str(I_Str'First)'Address );
   END CP;

   FUNCTION   CPS   ( I_Str : Win32.CHAR_Array )  RETURN  Win32.LPSTR  IS
      FUNCTION   UC  IS
         new Ada.Unchecked_Conversion ( System.Address , Win32.LPSTR );
   BEGIN
      RETURN  UC ( I_Str(I_Str'First)'Address );
   END CPS;

   PROCEDURE   StrCatFloat   ( I_Dest : Win32.PCHAR
                             ; I_Number : Float )  IS
      USE TYPE   Interfaces.C.size_t ;

      L_IgnoredP : Win32.PSTR ;

      Default_Fore : Integer := 1 ;
      Default_Aft  : Integer := Float'Digits - 1 ;
      Default_Exp  : Integer := 3 ;

      L_StringBuf : String(1..100);
      L_StringCnt : Integer ;
   BEGIN
      L_StringCnt := 0 ;
      System.Img_Real.Set_Image_Real ( Standard.LONG_LONG_FLOAT(I_Number)
                                     , L_StringBuf , L_StringCnt
                                     , Fore => Default_Fore , Aft => Default_Aft , Exp => Default_Exp );

      L_IgnoredP := Win32.Crt.Strings.StrNCat ( I_Dest , SCP(L_StringBuf) , Interfaces.C.size_t(L_StringCnt) );
   END StrCatFloat;

   USE TYPE   Win32.CHAR_Array ;      --- To get "&"

   Nul : Win32.Char RENAMES Win32.Nul ;

   IgnoredP : Win32.PSTR ;

   wm_MyAction : CONSTANT := Win32.Winuser.WM_USER + 101 ;

   C_MainMsg : CONSTANT Win32.CHAR_Array := "PMCalc..., "  &  Nul ;

   C_MainPMTitle : CONSTANT Win32.CHAR_Array := "Information"  &  Nul ;
   C_sRadiusIs   : CONSTANT Win32.CHAR_Array := "Radius="  &  Nul ;
   C_sCircumfIs  : CONSTANT Win32.CHAR_Array := "Circumference="  &  Nul ;
   C_sAreaIs     : CONSTANT Win32.CHAR_Array := "Area="  &  Nul ;
   CRLFz : CONSTANT Win32.CHAR_Array := Win32.Char(Character'Val(13))  &  Win32.Char(Character'Val(10))  &  Nul ;

   L_PaintStruct : ALIASED Win32.Winuser.PAINTSTRUCT ;
   L_Rect        : ALIASED Win32.Windef.RECT := (0,0,0,0);

   L_Buffer : ALIASED Win32.CHAR_Array(0..255);

   FUNCTION   MainWindow_Proc   ( I_HWnd    : Win32.Windef.HWND
                                ; I_Message : Win32.UINT
                                ; I_WParam  : Win32.WPARAM
                                ; I_LParam  : Win32.LPARAM )  RETURN  Win32.LRESULT  IS
      L_HDC : Win32.Windef.HDC ;

      PI : CONSTANT Float := 3.14159 ;

      R : Float ;
      C , A : Float ;
   BEGIN
      CASE I_Message IS
         WHEN wm_MyAction =>
            R := 10.0 ;

            C := 2.0 * PI * R ;
            A := PI * R * R ;

            L_Buffer(0) := Nul ;

            IgnoredP := Win32.Crt.Strings.StrCat ( CPS(L_Buffer) , CP(C_sRadiusIs) );
            StrCatFloat ( CPS(L_Buffer) , R );
            IgnoredP := Win32.Crt.Strings.StrCat ( CPS(L_Buffer) , CP(CRLFz) );

            IgnoredP := Win32.Crt.Strings.StrCat ( CPS(L_Buffer) , CP(C_sCircumfIs) );
            StrCatFloat ( CPS(L_Buffer) , C );
            IgnoredP := Win32.Crt.Strings.StrCat ( CPS(L_Buffer) , CP(CRLFz) );

            IgnoredP := Win32.Crt.Strings.StrCat ( CPS(L_Buffer) , CP(C_sAreaIs) );
            StrCatFloat ( CPS(L_Buffer) , A );
            IgnoredP := Win32.Crt.Strings.StrCat ( CPS(L_Buffer) , CP(CRLFz) );

            IgnoredI := Win32.Winuser.MessageBox ( I_HWnd
                                                 , CP(L_Buffer)
                                                 , CP(C_MainPMTitle)
                                                 , Win32.Winuser.MB_OK or Win32.Winuser.MB_ICONEXCLAMATION );

         WHEN Win32.Winuser.WM_PAINT =>
            L_HDC := Win32.Winuser.BeginPaint ( I_HWnd , L_PaintStruct'Access );
            IgnoredB := Win32.Winuser.GetClientRect ( I_HWnd , L_Rect'Access );

            IgnoredI := Win32.Winuser.DrawText ( L_HDC
                                               , CP(C_MainMsg)
                                               , Interfaces.C.Int( Win32.Crt.Strings.StrLen(CP(C_MainMsg)) )
                                               , L_Rect'Access
                                               , Win32.Winuser.DT_WORDBREAK );

            IgnoredB := Win32.Winuser.EndPaint ( L_HDC , L_PaintStruct'Access );

            ----- Only once...
            IF not G_MainPaintedOnceQ THEN
               G_MainPaintedOnceQ := Standard.TRUE ;
               IgnoredB := Win32.Winuser.PostMessage ( I_HWnd , wm_MyAction , 0 , 0 );      --- "Indirect" call...
            END IF;

         WHEN Win32.Winuser.WM_SIZE =>      --- When window is resized,...
            ----- Force paint again
            IgnoredB := Win32.Winuser.GetClientRect ( I_HWnd , L_Rect'Access );
            IgnoredB := Win32.Winuser.InvalidateRect ( I_HWnd , L_Rect'Access , Win32.FALSE );

         WHEN Win32.Winuser.WM_DESTROY =>
            Win32.Winuser.PostQuitMessage(0);

         WHEN OTHERS =>
            RETURN  Win32.Winuser.DefWIndowProc ( I_HWnd , I_Message , I_WParam , I_LParam );
      END CASE;

      ----- Default result : Msg processed!!!
      RETURN  0 ;
   END MainWindow_Proc;
END PMCalc_Pkg;
